#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (C) 2013 Pascal Billot skaalil@tuxfamily.org
# 
# This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

import re
import os
import tempfile
import shutil

import pygtk
pygtk.require("2.0")
import gtk
from subprocess import call
 

class LilySplit:
  def __init__(self):
    
    path = os.path.dirname(__file__)
    print path
    interface = gtk.Builder()
    interface.add_from_file(os.path.join(path,'lily_split.glade'))
    
    self.main_buffer = interface.get_object("main_code").get_buffer()
    self.voice_buffer = interface.get_object("voice_code").get_buffer()
    self.main_image = interface.get_object("main_image")
    self.voice_image = interface.get_object("voice_image")
    self.liste_boutons = []
    for index in range(5):
      n = "bouton%i"%index
      self.liste_boutons.append(interface.get_object(n))
    
    self.lily = Lily()
    self.fichiers_crees=[]
    interface.connect_signals(self)
    self.my_tmp_dir = tempfile.mkdtemp()
    os.chdir(self.my_tmp_dir)
    print os.getcwd()

  def on_main_window_destroy(self, widget):
    print "destruction..."
    shutil.rmtree(self.my_tmp_dir)
    gtk.main_quit()
    
  def launch_lilypond(self, code):
    fichier_ly = open("preview-%s.ly"%self.current_voice,'w')
    fichier_ly.write(code)
    
    fichier_ly.close()
    commande = ['lilypond','--png','-dpaper-size="a5"','-dresolution=60']
    commande.append("preview-%s.ly"%self.current_voice)
    call(commande)
    
  def send_to_image(self,t):
    
    
    if t=="main":
      self.main_image.set_from_file("preview-main.png")
    else:
      nom_fichier_png = "preview-%s.png"%self.current_voice
    
      self.voice_image.set_from_file(nom_fichier_png)
  
    
  def on_split_clicked(self, widget):
    
    #recupere le contenu de main_buffer
    start = self.main_buffer.get_start_iter()
    end = self.main_buffer.get_end_iter()
    texte = self.main_buffer.get_text(start,end)
    
    self.lily.set_code(texte)
    self.lily_voices_list = self.lily.split_chords()

    for bouton in self.liste_boutons:
      bouton.hide()
    #affiche le nombre de boutons égal au nombre de voix
    b=iter(self.liste_boutons)
    for n in self.lily_voices_list:
      b.next().show()
    if len(self.lily_voices_list) > 0 :
      self.on_bouton0_clicked(None)
      
  def on_main_view_clicked(self, widget):
    
    start = self.main_buffer.get_start_iter()
    end = self.main_buffer.get_end_iter()
    texte = self.main_buffer.get_text(start,end)
    clef = "treble"
    code = r'\version "2.12.3" \layout {{ indent = 0.0\cm }} {{ \new PianoStaff \autochange {{ {0} }} }}'
    
    code = code.format(texte)
    
    fichier_ly = open("preview-main.ly",'w')
    fichier_ly.write(code)
    fichier_ly.close()
    commande = ['lilypond','--png','-dpaper-size="a5"','-dresolution=60']
    commande.append("preview-main.ly")
    call(commande)
    
    self.send_to_image('main')
      
  def on_bouton0_clicked(self, widget):
    self.voice_buffer.set_text(self.lily_voices_list[0])
    self.current_voice = 0
    self.send_to_image("voice")
         
  def on_bouton1_clicked(self, widget):
    self.voice_buffer.set_text(self.lily_voices_list[1])
    self.current_voice = 1
    self.send_to_image("voice")
    
  def on_bouton2_clicked(self, widget):
    self.voice_buffer.set_text(self.lily_voices_list[2])
    self.current_voice = 2
    self.send_to_image("voice")
    
  def on_bouton3_clicked(self, widget):
    self.voice_buffer.set_text(self.lily_voices_list[3])
    self.current_voice = 3
    self.send_to_image("voice")

  def on_bouton4_clicked(self, widget):
    self.voice_buffer.set_text(self.lily_voices_list[4])
    self.current_voice = 4
    self.send_to_image("voice")
    
  def on_bouton5_clicked(self, widget):
    self.voice_buffer.set_text(self.lily_voices_list[5])
    self.current_voice = 5
    self.send_to_image("voice")
    
  def on_voice_g_clicked(self, widget):
    clef = "treble"
    code = r'\version "2.12.3" \layout {{ indent = 0.0\cm }} {{ \clef {0} {1} }}'
    code = code.format(clef, self.lily_voices_list[self.current_voice])
    nom_fichier = "preview%i"%self.current_voice
    self.launch_lilypond(code)
    self.send_to_image('voice')
    
  def on_voice_c_clicked(self, widget):
    clef = "alto"
    code = r'\version "2.12.3" \layout {{ indent = 0.0\cm }} {{ \clef {0} {1} }}'
    code = code.format(clef, self.lily_voices_list[self.current_voice])
    nom_fichier = "preview%i"%self.current_voice
    self.launch_lilypond(code)
    self.send_to_image('voice')
    
  def on_voice_f_clicked(self, widget):
    clef = "bass"
    code = r'\version "2.12.3" \layout {{ indent = 0.0\cm }} {{ \clef {0} {1} }}'
    code = code.format(clef, self.lily_voices_list[self.current_voice])
    nom_fichier = "preview%i"%self.current_voice
    self.launch_lilypond(code)
    self.send_to_image('voice')
    
class Lily:
  
  def __init__(self):
    self.code = ''
  
  def set_code(self, code):
    self.code = code
    
  def replace_chord(self,pat):
    sortie = "{%i}"%self.i
    self.i+=1
    return sortie
    
  def split_chords(self):
    #remplace les accords par {}
    self.i = 0
    code_prov = re.sub(r"<[ a-z,']+>",self.replace_chord,self.code)
    self.i = 0
    #print code_prov
    # extrait les accords du code
    chords = re.findall(r"<[ a-z,']+>", self.code)
    # un peu de nettoyage, on enlève les < et les >
    chords = [re.sub(r"[<>]",'',c) for c in chords]
    
    list_chords = [i.split() for i in chords]
    voices_list_var = zip(*list_chords)
    voices_list = [code_prov.format(*var) for var in voices_list_var]
    voices_list.reverse()
    return voices_list
    
if __name__ == "__main__":

	LilySplit()
	gtk.main()
  
